using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.Windows.Forms.Design;

namespace Cyotek.Windows.Forms.ToolStripControllerHosts
{
  [DefaultProperty("Value"), DefaultEvent("ValueChanged")]
  [ToolStripItemDesignerAvailability(ToolStripItemDesignerAvailability.ToolStrip | ToolStripItemDesignerAvailability.StatusStrip)]
  public partial class ToolStripTrackBar : ToolStripControlHost
  {
    #regionPublicConstructors

    public ToolStripTrackBar()
      : base(ToolStripTrackBar.CreateControlInstance())
    {
      this.Maximum = 10;
      this.Minimum = 0;
      this.TickFrequency = 1;
      this.Value = 0;
      this.LargeChange = 5;
      this.SmallChange = 1;
      this.TickStyle = TickStyle.BottomRight;
      this.AutoToolTip = true;
    }

    #endregionPublicConstructors

    #regionEvents

    public event EventHandler ValueChanged;

    #endregionEvents

    #regionPrivateClassMethods

    private static Control CreateControlInstance()
    {
      TrackBar child;

      child = new TrackBar();
      child.AutoSize = false;
      child.Height = 16;

      return child;
    }

    #endregionPrivateClassMethods

    #regionOverridenProperties

    protected override Size DefaultSize
    { get { return new Size(200, 16); } }

    [Browsable(false)]
    public override string Text
    {
      get { return base.Text; }
      set { base.Text = value; }
    }

    #endregionOverridenProperties

    #regionProtectedOverriddenMethods

    protected override void OnBoundsChanged()
    {
      base.OnBoundsChanged();

      this.TrackBar.Size = this.Bounds.Size;
    }

    protected override void OnSubscribeControlEvents(Control control)
    {
      TrackBar child;

      child = control as TrackBar;
      child.ValueChanged += this.TrackbarValueChangedHandler;

      base.OnSubscribeControlEvents(control);
    }

    protected override void OnUnsubscribeControlEvents(Control control)
    {
      TrackBar child;

      child = control as TrackBar;
      child.ValueChanged -= this.TrackbarValueChangedHandler;

      base.OnUnsubscribeControlEvents(control);
    }

    #endregionProtectedOverriddenMethods

    #regionPublicProperties

    [DefaultValue(true)]
    public new bool AutoToolTip
    {
      get { return base.AutoToolTip; }
      set { base.AutoToolTip = value; }
    }

    [DefaultValue(5), Category("Behavior")]
    public int LargeChange
    {
      get { return this.TrackBar.LargeChange; }
      set { this.TrackBar.LargeChange = value; }
    }

    [DefaultValue(10), Category("Behavior")]
    public int Maximum
    {
      get { return this.TrackBar.Maximum; }
      set { this.TrackBar.Maximum = value; }
    }

    [DefaultValue(0), Category("Behavior")]
    public int Minimum
    {
      get { return this.TrackBar.Minimum; }
      set { this.TrackBar.Minimum = value; }
    }

    [DefaultValue(1), Category("Behavior")]
    public int SmallChange
    {
      get { return this.TrackBar.SmallChange; }
      set { this.TrackBar.SmallChange = value; }
    }

    [DefaultValue(1), Category("Appearance")]
    public int TickFrequency
    {
      get { return this.TrackBar.TickFrequency; }
      set { this.TrackBar.TickFrequency = value; }
    }

    [DefaultValue(typeof(TickStyle), "BottomRight"), Category("Appearance")]
    public TickStyle TickStyle
    {
      get { return this.TrackBar.TickStyle; }
      set { this.TrackBar.TickStyle = value; }
    }

    [Browsable(false)]
    public TrackBar TrackBar
    { get { return Control as TrackBar; } }

    [DefaultValue(0), Category("Behavior")]
    public int Value
    {
      get { return this.TrackBar.Value; }
      set { this.TrackBar.Value = value; }
    }

    #endregionPublicProperties

    #regionPrivateMethods

    private void TrackbarValueChangedHandler(object sender, EventArgs e)
    {
      this.OnValueChanged(EventArgs.Empty);
    }

    #endregionPrivateMethods

    #regionProtectedMethods

    protected virtual void OnValueChanged(EventArgs e)
    {
      EventHandler handler;

      this.Text = this.Value.ToString();

      handler = this.ValueChanged;

      if (handler != null)
        handler(this, e);
    }

    #endregionProtectedMethods
  }
}
